@@ -285,7 +285,8 @@ static int procsearch(const char *dbname, CBMAP *params, const char *outfile,
   rateuri = rateuri && !(phrase && cbstrfwmatch(phrase, ESTOPSIMILAR));
   for(i = 0; i < rnum && cnt < lmax; i++){
     if(!(doc = est_db_get_doc(db, res[i], 0))) continue;
-    if(scancheck && phrase && phrase[0] != '[' && !est_db_scan_doc(db, doc, cond)){
+    if(scancheck && phrase && phrase[0] != '[' && phrase[0] != '*' &&
+       !est_db_scan_doc(db, doc, cond)){
       est_doc_delete(doc);
       continue;
     }
