@@ -499,7 +499,7 @@ static void sendnodecmdsearch(ESTMTDB *db, const char *myurl, const char *mylabe
   rateuri = g_rateuri && !cbstrfwmatch(phrase, ESTOPSIMILAR);
   for(i = 0; i < rnum && cnt < lmax; i++){
     if(!(doc = est_mtdb_get_doc(db, res[i], 0))) continue;
-    if(g_scancheck && *phrase != '\0' && *phrase != '[' &&
+    if(g_scancheck && *phrase != '\0' && *phrase != '[' && *phrase != '*' &&
        res[i] < ESTPDOCIDMIN && !est_mtdb_scan_doc(db, doc, cond)){
       est_doc_delete(doc);
       miss++;
