@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of the node API
- *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2007 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -532,7 +532,7 @@ int est_node_out_doc(ESTNODE *node, int id){
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  est_datum_printf(reqbody, "id=%d", id);
+  cbdatumprintf(reqbody, "id=%d", id);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, NULL)){
@@ -570,7 +570,7 @@ int est_node_out_doc_by_uri(ESTNODE *node, const char *uri){
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  est_datum_printf(reqbody, "uri=%?", uri);
+  cbdatumprintf(reqbody, "uri=%?", uri);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, NULL)){
@@ -656,7 +656,7 @@ ESTDOC *est_node_get_doc(ESTNODE *node, int id){
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  est_datum_printf(reqbody, "id=%d", id);
+  cbdatumprintf(reqbody, "id=%d", id);
   resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
@@ -698,7 +698,7 @@ ESTDOC *est_node_get_doc_by_uri(ESTNODE *node, const char *uri){
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  est_datum_printf(reqbody, "uri=%?", uri);
+  cbdatumprintf(reqbody, "uri=%?", uri);
   resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
@@ -739,7 +739,7 @@ char *est_node_get_doc_attr(ESTNODE *node, int id, const char *name){
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  est_datum_printf(reqbody, "id=%d&attr=%?", id, name);
+  cbdatumprintf(reqbody, "id=%d&attr=%?", id, name);
   resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
@@ -785,7 +785,7 @@ char *est_node_get_doc_attr_by_uri(ESTNODE *node, const char *uri, const char *n
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  est_datum_printf(reqbody, "uri=%?&attr=%?", uri, name);
+  cbdatumprintf(reqbody, "uri=%?&attr=%?", uri, name);
   resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
@@ -833,7 +833,7 @@ CBMAP *est_node_etch_doc(ESTNODE *node, int id){
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  est_datum_printf(reqbody, "id=%d", id);
+  cbdatumprintf(reqbody, "id=%d", id);
   resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
@@ -887,7 +887,7 @@ CBMAP *est_node_etch_doc_by_uri(ESTNODE *node, const char *uri){
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  est_datum_printf(reqbody, "uri=%?", uri);
+  cbdatumprintf(reqbody, "uri=%?", uri);
   resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
@@ -940,7 +940,7 @@ int est_node_uri_to_id(ESTNODE *node, const char *uri){
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  est_datum_printf(reqbody, "uri=%?", uri);
+  cbdatumprintf(reqbody, "uri=%?", uri);
   resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
@@ -1093,35 +1093,35 @@ ESTNODERES *est_node_search(ESTNODE *node, ESTCOND *cond, int depth){
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  if((phrase = est_cond_phrase(cond)) != NULL) est_datum_printf(reqbody, "phrase=%?", phrase);
+  if((phrase = est_cond_phrase(cond)) != NULL) cbdatumprintf(reqbody, "phrase=%?", phrase);
   if((attrs = est_cond_attrs(cond)) != NULL){
     for(i = 0; i < cblistnum(attrs); i++){
       if(cbdatumsize(reqbody) > 0) cbdatumcat(reqbody, "&", 1);
-      est_datum_printf(reqbody, "attr%d=%?", i + 1, cblistval(attrs, i, NULL));
+      cbdatumprintf(reqbody, "attr%d=%?", i + 1, cblistval(attrs, i, NULL));
     }
   }
   if((max = est_cond_max(cond)) >= 0){
     if(cbdatumsize(reqbody) > 0) cbdatumcat(reqbody, "&", 1);
-    est_datum_printf(reqbody, "max=%d", max);
+    cbdatumprintf(reqbody, "max=%d", max);
   } else {
     if(cbdatumsize(reqbody) > 0) cbdatumcat(reqbody, "&", 1);
-    est_datum_printf(reqbody, "max=%d", INT_MAX / 2);
+    cbdatumprintf(reqbody, "max=%d", INT_MAX / 2);
   }
   if((order = est_cond_order(cond)) != NULL){
     if(cbdatumsize(reqbody) > 0) cbdatumcat(reqbody, "&", 1);
-    est_datum_printf(reqbody, "order=%?", order);
+    cbdatumprintf(reqbody, "order=%?", order);
   }
   if(cbdatumsize(reqbody) > 0) cbdatumcat(reqbody, "&", 1);
-  est_datum_printf(reqbody, "options=%d", est_cond_options(cond));
-  est_datum_printf(reqbody, "&auxiliary=%d", est_cond_auxiliary(cond));
+  cbdatumprintf(reqbody, "options=%d", est_cond_options(cond));
+  cbdatumprintf(reqbody, "&auxiliary=%d", est_cond_auxiliary(cond));
   if((distinct = est_cond_distinct(cond)) != NULL)
-    est_datum_printf(reqbody, "&distinct=%?", distinct);
-  est_datum_printf(reqbody, "&depth=%d", depth);
-  est_datum_printf(reqbody, "&wwidth=%d", node->wwidth);
-  est_datum_printf(reqbody, "&hwidth=%d", node->hwidth);
-  est_datum_printf(reqbody, "&awidth=%d", node->awidth);
-  est_datum_printf(reqbody, "&skip=%d", est_cond_skip(cond));
-  est_datum_printf(reqbody, "&mask=%d", est_cond_mask(cond));
+    cbdatumprintf(reqbody, "&distinct=%?", distinct);
+  cbdatumprintf(reqbody, "&depth=%d", depth);
+  cbdatumprintf(reqbody, "&wwidth=%d", node->wwidth);
+  cbdatumprintf(reqbody, "&hwidth=%d", node->hwidth);
+  cbdatumprintf(reqbody, "&awidth=%d", node->awidth);
+  cbdatumprintf(reqbody, "&skip=%d", est_cond_skip(cond));
+  cbdatumprintf(reqbody, "&mask=%d", est_cond_mask(cond));
   resheads = cbmapopenex(ESTMINIBNUM);
   resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(buf, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
@@ -1225,7 +1225,7 @@ int est_node_set_user(ESTNODE *node, const char *name, int mode){
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  est_datum_printf(reqbody, "name=%?&mode=%d", name, mode);
+  cbdatumprintf(reqbody, "name=%?&mode=%d", name, mode);
   resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
@@ -1266,9 +1266,9 @@ int est_node_set_link(ESTNODE *node, const char *url, const char *label, int cre
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
   if(credit >= 0){
-    est_datum_printf(reqbody, "url=%?&label=%?&credit=%d", url, label, credit);
+    cbdatumprintf(reqbody, "url=%?&label=%?&credit=%d", url, label, credit);
   } else {
-    est_datum_printf(reqbody, "url=%?&label=%?", url, label);
+    cbdatumprintf(reqbody, "url=%?&label=%?", url, label);
   }
   resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(myurl, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
@@ -1551,88 +1551,6 @@ void est_sock_send_all(int sock, const char *buf, int len){
 }
 
 
-/* Perform formatted output into a datum object. */
-void est_datum_printf(CBDATUM *datum, const char *format, ...){
-  va_list ap;
-  char *tmp, cbuf[ESTNUMBUFSIZ], tbuf[ESTNUMBUFSIZ*2];
-  unsigned char c;
-  int cblen, tlen;
-  assert(datum && format);
-  va_start(ap, format);
-  while(*format != '\0'){
-    if(*format == '%'){
-      cbuf[0] = '%';
-      cblen = 1;
-      format++;
-      while(strchr("0123456789 .+-", *format) && *format != '\0' && cblen < ESTNUMBUFSIZ - 1){
-        cbuf[cblen++] = *format;
-        format++;
-      }
-      cbuf[cblen++] = *format;
-      cbuf[cblen] = '\0';
-      switch(*format){
-      case 's':
-        tmp = va_arg(ap, char *);
-        if(!tmp) tmp = "(null)";
-        cbdatumcat(datum, tmp, -1);
-        break;
-      case 'd':
-        tlen = sprintf(tbuf, cbuf, va_arg(ap, int));
-        cbdatumcat(datum, tbuf, tlen);
-        break;
-      case 'o': case 'u': case 'x': case 'X': case 'c':
-        tlen = sprintf(tbuf, cbuf, va_arg(ap, unsigned int));
-        cbdatumcat(datum, tbuf, tlen);
-        break;
-      case 'e': case 'E': case 'f': case 'g': case 'G':
-        tlen = sprintf(tbuf, cbuf, va_arg(ap, double));
-        cbdatumcat(datum, tbuf, tlen);
-        break;
-      case '@':
-        tmp = va_arg(ap, char *);
-        if(!tmp) tmp = "(null)";
-        while(*tmp){
-          switch(*tmp){
-          case '&': cbdatumcat(datum, "&amp;", 5); break;
-          case '<': cbdatumcat(datum, "&lt;", 4); break;
-          case '>': cbdatumcat(datum, "&gt;", 4); break;
-          case '"': cbdatumcat(datum, "&quot;", 6); break;
-          default:
-            if(!((*tmp >= 0 && *tmp <= 0x8) || (*tmp >= 0x0e && *tmp <= 0x1f)))
-              cbdatumcat(datum, tmp, 1);
-            break;
-          }
-          tmp++;
-        }
-        break;
-      case '?':
-        tmp = va_arg(ap, char *);
-        if(!tmp) tmp = "(null)";
-        while(*tmp){
-          c = *(unsigned char *)tmp;
-          if((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') ||
-             (c >= '0' && c <= '9') || (c != '\0' && strchr("_-.", c))){
-            cbdatumcat(datum, tmp, 1);
-          } else {
-            tlen = sprintf(tbuf, "%%%02X", c);
-            cbdatumcat(datum, tbuf, tlen);
-          }
-          tmp++;
-        }
-        break;
-      case '%':
-        cbdatumcat(datum, "%", 1);
-        break;
-      }
-    } else {
-      cbdatumcat(datum, format, 1);
-    }
-    format++;
-  }
-  va_end(ap);
-}
-
-
 /* Perform an interaction of a URL. */
 int est_url_shuttle(const char *url, const char *pxhost, int pxport, int outsec, int limsize,
                     const char *auth, const CBLIST *reqheads, const char *reqbody, int rbsiz,
@@ -1932,32 +1850,32 @@ static void *est_url_shuttle_impl(void *targ){
   datum = cbdatumopen(NULL, -1);
   pthread_cleanup_push((void (*)(void *))cbdatumclose, datum);
   if(pxhost){
-    est_datum_printf(datum, "%s %s HTTP/1.0\r\n", reqbody ? "POST" : "GET", url);
+    cbdatumprintf(datum, "%s %s HTTP/1.0\r\n", reqbody ? "POST" : "GET", url);
   } else if(reqbody){
-    est_datum_printf(datum, "POST %s HTTP/1.0\r\n", path);
+    cbdatumprintf(datum, "POST %s HTTP/1.0\r\n", path);
   } else if(query[0] != 0){
-    est_datum_printf(datum, "GET %s?%s HTTP/1.0\r\n", path, query);
+    cbdatumprintf(datum, "GET %s?%s HTTP/1.0\r\n", path, query);
   } else {
-    est_datum_printf(datum, "GET %s HTTP/1.0\r\n", path);
+    cbdatumprintf(datum, "GET %s HTTP/1.0\r\n", path);
   }
-  est_datum_printf(datum, "Host: %s:%d\r\n", host, port);
-  est_datum_printf(datum, "Connection: close\r\n", path);
-  est_datum_printf(datum, "User-Agent: %s/%s\r\n", ESTAGENTNAME, est_version);
+  cbdatumprintf(datum, "Host: %s:%d\r\n", host, port);
+  cbdatumprintf(datum, "Connection: close\r\n", path);
+  cbdatumprintf(datum, "User-Agent: %s/%s\r\n", ESTAGENTNAME, est_version);
   if(auth){
     enc = cbbaseencode(auth, -1);
-    est_datum_printf(datum, "Authorization: Basic %s\r\n", enc);
+    cbdatumprintf(datum, "Authorization: Basic %s\r\n", enc);
     free(enc);
   }
-  if(ESTUSEZLIB) est_datum_printf(datum, "Accept-Encoding: deflate\r\n");
-  if(reqbody) est_datum_printf(datum, "Content-Length: %d\r\n", rbsiz);
+  if(ESTUSEZLIB) cbdatumprintf(datum, "Accept-Encoding: deflate\r\n");
+  if(reqbody) cbdatumprintf(datum, "Content-Length: %d\r\n", rbsiz);
   if(reqheads){
     for(i = 0; i < cblistnum(reqheads); i++){
       rp = cblistval(reqheads, i, &size);
-      est_datum_printf(datum, rp, size);
-      est_datum_printf(datum, "\r\n", 2);
+      cbdatumprintf(datum, rp, size);
+      cbdatumprintf(datum, "\r\n", 2);
     }
   }
-  est_datum_printf(datum, "\r\n");
+  cbdatumprintf(datum, "\r\n");
   est_sock_send_all(sock, cbdatumptr(datum), cbdatumsize(datum));
   if(reqbody) est_sock_send_all(sock, reqbody, rbsiz);
   if((size = est_sock_recv_line(sock, iobuf, ESTIOBUFSIZ - 1)) < 1 ||
@@ -2188,8 +2106,8 @@ static int est_node_read_list(ESTNODE *node){
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen(NULL, -1);
-  est_datum_printf(reqbody, "max=%d", ESTITERUNIT);
-  if(node->iprev) est_datum_printf(reqbody, "&prev=%?", node->iprev);
+  cbdatumprintf(reqbody, "max=%d", ESTITERUNIT);
+  if(node->iprev) cbdatumprintf(reqbody, "&prev=%?", node->iprev);
   resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
