@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * A proxy with highlighting
- *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2007 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -115,11 +115,9 @@ int main(int argc, char **argv){
   est_proc_env_reset();
   est_init_net_env();
   while(FCGI_Accept() >= 0){
-    if(++cnt >= 256){
-      cbggcsweep();
-      cnt = 0;
-    }
     realmain(argc, argv);
+    fflush(stdout);
+    if(++cnt >= 256) exit(0);
   }
   est_free_net_env();
   return 0;
@@ -425,7 +423,7 @@ static void xmlprintf(const char *format, ...){
           default:
             if(!((*tmp >= 0 && *tmp <= 0x8) || (*tmp >= 0x0e && *tmp <= 0x1f))) putchar(*tmp);
             break;
-            }
+          }
           tmp++;
         }
         if(ebuf){
@@ -488,7 +486,7 @@ static void sendform(void){
   printf("\r\n");
   xmlprintf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
   xmlprintf("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
-         " \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n");
+            " \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n");
   xmlprintf("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n");
   xmlprintf("<head>\n");
   xmlprintf("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\n");
@@ -639,7 +637,7 @@ static void senddirlist(const char *path){
   printf("\r\n");
   xmlprintf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
   xmlprintf("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
-         " \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n");
+            " \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n");
   xmlprintf("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"%@\" lang=\"%p\">\n",
             g_language, g_language);
   xmlprintf("<head>\n");
@@ -934,9 +932,9 @@ static void modifylocation(CBMAP *heads, const char *name){
   if(!(url = cbmapget(heads, name, -1, NULL))) return;
   nurl = cburlresolve(p_url, url);
   datum = cbdatumopen(NULL, -1);
-  est_datum_printf(datum, "http://%s%s?url=%?&once=%d", g_hostname, g_scriptpath, nurl, p_once);
+  cbdatumprintf(datum, "http://%s%s?url=%?&once=%d", g_hostname, g_scriptpath, nurl, p_once);
   for(i = 0; i < cblistnum(p_words); i++){
-    est_datum_printf(datum, "&word%d=%?", i + 1, cblistval(p_words, i, NULL));
+    cbdatumprintf(datum, "&word%d=%?", i + 1, cblistval(p_words, i, NULL));
   }
   cbmapput(heads, name, -1, cbdatumptr(datum), cbdatumsize(datum), TRUE);
   cbdatumclose(datum);
@@ -1280,7 +1278,7 @@ static void sendpagefromtext(const char *buf, int size, const char *penc){
   } else {
     text = buf;
     nbuf = est_iconv(buf, size, enc, "UTF-8", NULL, NULL);
-      if(nbuf) text = nbuf;
+    if(nbuf) text = nbuf;
   }
   xmlprintf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
   xmlprintf("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
