@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The core API of Hyper Estraier
- *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2007 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -268,6 +268,8 @@ typedef struct {                         /* type of structure for search conditi
   int scfb;                              /* whether to feed back scores */
   int *scores;                           /* array of scores */
   int snum;                              /* number of elemnts of the score array */
+  const int *nscores;                    /* array of narrowing scores */
+  int nsnum;                             /* number of elemnts of the narrowing score array */
   int opts;                              /* options for preservation */
   double ecllim;                         /* lower limit of similarity eclipse */
   CBMAP *shadows;                        /* map of eclipsed documents */
@@ -519,7 +521,7 @@ const char *est_err_msg(int ecode);
    `ESTDBLCKNB', which means locking is performed without blocking.  If `ESTDBNOLCK' is used,
    the application is responsible for exclusion control.  `ESTDBCREAT' can be added to by bitwise
    or: `ESTDBPERFNG', which means N-gram analysis is performed against European text also,
-   `ESTDBCHACAT', which means character category analysis is performed instead of N-gram analysis,
+   `ESTDBCHRCAT', which means character category analysis is performed instead of N-gram analysis,
    `ESTDBSMALL', which means the index is tuned to register less than 50000 documents,
    `ESTDBLARGE', which means the index is tuned to register more than 300000 documents,
    `ESTDBHUGE', which means the index is tuned to register more than 1000000 documents,
@@ -747,13 +749,13 @@ int est_db_add_pseudo_index(ESTDB *db, const char *path);
  *************************************************************************************************/
 
 
-#define _EST_VERSION   "1.4.9"
-#define _EST_LIBVER    834
+#define _EST_VERSION   "1.4.13"
+#define _EST_LIBVER    838
 #define _EST_PROTVER   "1.0"
 
-#define _EST_PROJURL   "http://hyperestraier.sourceforge.net/"
-#define _EST_XNSEARCH  "http://hyperestraier.sourceforge.net/xmlns/search"
-#define _EST_XNNODE    "http://hyperestraier.sourceforge.net/xmlns/node"
+#define _EST_PROJURL   "http://fallabs.com/hyperestraier/"
+#define _EST_XNSEARCH  "http://fallabs.com/hyperestraier/xmlns/search"
+#define _EST_XNNODE    "http://fallabs.com/hyperestraier/xmlns/node"
 
 enum {                                   /* enumeration for languages */
   ESTLANGEN,                             /* English */
@@ -774,6 +776,18 @@ enum {                                   /* enumeration for database repair */
   ESTRPSHODDY = 1 << 1                   /* omit consistency check */
 };
 
+typedef struct {                         /* type of structure for an element of result map */
+  const char *key;                       /* pointer to the key string */
+  int score;                             /* total score */
+} ESTRESMAPELEM;
+
+enum {                                   /* enumeration for scoring for result map */
+  ESTRMLOSUM,                            /* summation */
+  ESTRMLOMAX,                            /* maximum */
+  ESTRMLOMIN,                            /* minimum */
+  ESTRMLOAVG                             /* average */
+};
+
 
 /* Break a sentence of text and extract words.
    `text' specifies a sentence of text.
@@ -1116,6 +1130,22 @@ int est_cond_mask(ESTCOND *cond);
 int est_cond_score(ESTCOND *cond, int index);
 
 
+/* Get the score array of corresponding documents of a condition object.
+   `cond' specifies a condition object.
+   `nump' specifies the pointer to a variable to which the number of elements in the score array
+   is assigned.
+   The return value is the score array of corresponding documents. */
+const int *est_cond_scores(ESTCOND *cond, int *nump);
+
+
+/* Set the narrowing scores of a condition object.
+   `cond' specifies a condition object.
+   `scores' specifies the pointer to an array of narrowing scores.  The life duration of the
+   array should be equal to or longer than the condition object itself.
+   `num' specifies the number of the array. */
+void est_cond_set_narrowing_scores(ESTCOND *cond, const int *scores, int num);
+
+
 /* Check whether a condition object has used the auxiliary index.
    `cond' specifies a condition object.
    `word' specifies a keyword to be checked.  If it is an empty string, whether at least one
@@ -1429,6 +1459,26 @@ int est_db_repair(const char *name, int options, int *ecp);
 CBLIST *est_hints_to_words(CBMAP *hints);
 
 
+/* Add a record into a result map for logical operation.
+   `map' specifies a map object.
+   `key' specifies the key of a record.
+   `score' specifies the score of the record.
+   `method' specifies a scoring method when logical operation.  As for now, `ESTRMLOSUM',
+   `ESTRMLOMAX', `ESTRMLOMIN', and `ESTRMLOAVG'. */
+void est_resmap_add(CBMAP *map, const char *key, int score, int method);
+
+
+/* Dump a result list of a result map for logical operation.
+   `map' specifies a map object.
+   `min' specifies the minimum number of times for which each element of the result occurs.
+   `nump' specifies the pointer to a variable to which the number of elements in the result is
+   assigned.
+   The return value is an array whose elements are structures of keys and scores.  Because the
+   region of the return value is allocated with the `malloc' call, it should be released with the
+   `free' call if it is no longer in use. */
+ESTRESMAPELEM *est_resmap_dump(CBMAP *map, int min, int *nump);
+
+
 /* Reset the environment of the process.
    This function sets the standard streams as binary mode and resets environment variables for
    locale. */
@@ -1461,6 +1511,13 @@ char *est_realpath(const char *path);
 int est_inode(const char *path);
 
 
+/* Change modification time of a file.
+   `path' specifies the path of a file.
+   `mtime' specifies modification time.  If it is negative, the current time is set.
+   The return value is true if success, else it is false. */
+int est_utime(const char *path, time_t mtime);
+
+
 /* Get the time of day in milliseconds.
    The return value is the time of day in milliseconds. */
 double est_gettimeofday(void);
