@@ -59,8 +59,11 @@ int g_phraseform = 0;                    /* mode of phrase form */
 int g_wildmax = -1;                      /* maximum number of extension of wild cards */
 int g_stmode = FALSE;                    /* whether to be single thread mode */
 const char *g_idattr = NULL;             /* attribute for identification */
+const CBLIST *g_idsuflist = NULL;        /* list of suffixes for identification */
 const char *g_ordexpr = NULL;            /* attribute for ordering when attribute search */
 int g_dupcheck = FALSE;                  /* whether to check duplication of identifiers */
+int g_union = FALSE;                     /* whether to perform union meta search */
+int g_score = -1;                        /* scoring method when logical operation */
 const char *g_tmpdir = NULL;             /* path of the directory for temporary files */
 int g_cclife = 0;                        /* lifetime of cache files */
 const char *g_logfile = NULL;            /* path of the log file */
@@ -71,8 +74,10 @@ const char *g_logformat = NULL;          /* format of the log */
 const CBLIST *p_phraselist = NULL;       /* list of phrase conditions */
 const CBLIST *p_attrlist = NULL;         /* list of attribute conditions */
 int p_max = -1;                          /* number of output documents */
+const char *p_distinct = NULL;           /* name of distinction attribute */
 int p_fresh = FALSE;                     /* whether to retrieve fresh result */
 int p_stmode = FALSE;                    /* whether to be single thread mode */
+const char *p_logmsg = "";               /* additional log message */
 
 
 /* other global variables */
@@ -94,6 +99,7 @@ static CBMAP *getparameters(void);
 static void showresult(void);
 static void *procsearch(void *targ);
 static char *myencode(ESTCOND *cond);
+static char *caturisuffixes(ESTMTDB *db, int id, const char *uri);
 static void outputlog(void);
 
 
@@ -106,8 +112,10 @@ int main(int argc, char **argv){
     p_phraselist = NULL;
     p_attrlist = NULL;
     p_max = -1;
+    p_distinct = NULL;
     p_fresh = FALSE;
     p_stmode = FALSE;
+    p_logmsg = "";
     g_metamin = 0;
     g_sort = FALSE;
     g_hnum = 0;
@@ -141,7 +149,7 @@ int main(int argc, char **argv){
 
 /* main routine */
 static int realmain(int argc, char **argv){
-  CBLIST *lines, *ilist, *plist, *alist;
+  CBLIST *lines, *ilist, *slist, *plist, *alist;
   CBMAP *params;
   const char *rp;
   char *tmp, *wp, numbuf[NUMBUFSIZ];
@@ -161,6 +169,8 @@ static int realmain(int argc, char **argv){
   cbglobalgc(lines, (void (*)(void *))cblistclose);
   ilist = cblistopen();
   cbglobalgc(ilist, (void (*)(void *))cblistclose);
+  slist = cblistopen();
+  cbglobalgc(slist, (void (*)(void *))cblistclose);
   for(i = 0; i < cblistnum(lines); i++){
     rp = cblistval(lines, i, NULL);
     if(cbstrfwimatch(rp, "indexname:")){
@@ -182,10 +192,17 @@ static int realmain(int argc, char **argv){
       g_stmode = atoi(skiplabel(rp)) > 0;
     } else if(cbstrfwimatch(rp, "idattr:")){
       g_idattr = skiplabel(rp);
+    } else if(cbstrfwimatch(rp, "idsuffix:")){
+      rp = skiplabel(rp);
+      if(*rp != '\0') cblistpush(slist, rp, -1);
     } else if(cbstrfwimatch(rp, "ordexpr:")){
       g_ordexpr = skiplabel(rp);
     } else if(cbstrfwimatch(rp, "dupcheck:")){
       g_dupcheck = atoi(skiplabel(rp)) > 0;
+    } else if(cbstrfwimatch(rp, "union:")){
+      g_union = atoi(skiplabel(rp)) > 0;
+    } else if(cbstrfwimatch(rp, "score:")){
+      g_score = atoi(skiplabel(rp));
     } else if(cbstrfwimatch(rp, "tmpdir:")){
       g_tmpdir = skiplabel(rp);
     } else if(cbstrfwimatch(rp, "cclife:")){
@@ -203,7 +220,9 @@ static int realmain(int argc, char **argv){
   if(g_phraseform < 1) showerror("phraseform is undefined.");
   if(g_wildmax < 0) showerror("wildmax is undefined.");
   if(!g_idattr) showerror("idattr is undefined.");
+  g_idsuflist = slist;
   if(!g_ordexpr) showerror("ordexpr is undefined.");
+  if(g_score < 0) showerror("score is undefined.");
   if(!g_tmpdir) showerror("tmpdir is undefined.");
   if(!g_logfile) showerror("logfile is undefined.");
   if(!g_logformat) showerror("logformat is undefined.");
@@ -214,15 +233,25 @@ static int realmain(int argc, char **argv){
   cbglobalgc(plist, (void (*)(void *))cblistclose);
   alist = cblistopen();
   cbglobalgc(alist, (void (*)(void *))cblistclose);
-  for(i = 1; i <= cblistnum(g_indexlist); i++){
-    len = sprintf(numbuf, "phrase%d", i);
-    rp = cbmapget(params, numbuf, len, NULL);
-    if(!rp && i == 1) rp = cbmapget(params, "phrase", -1, NULL);
-    if(!rp) rp = "";
+  if(g_union && ((rp = cbmapget(params, "phrase", -1, NULL)) ||
+                 (rp = cbmapget(params, "phrase1", -1, NULL)))){
     while(*rp == ' ' || *rp == '\t'){
       rp++;
     }
-    cblistpush(plist, rp, -1);
+    for(i = 0; i < cblistnum(g_indexlist); i++){
+      cblistpush(plist, rp, -1);
+    }
+  } else {
+    for(i = 1; i <= cblistnum(g_indexlist); i++){
+      len = sprintf(numbuf, "phrase%d", i);
+      rp = cbmapget(params, numbuf, len, NULL);
+      if(!rp && i == 1) rp = cbmapget(params, "phrase", -1, NULL);
+      if(!rp) rp = "";
+      while(*rp == ' ' || *rp == '\t'){
+        rp++;
+      }
+      cblistpush(plist, rp, -1);
+    }
   }
   if((rp = cbmapget(params, "attr", -1, NULL)) != NULL) cblistpush(alist, rp, -1);
   for(i = 1; i <= CONDATTRMAX; i++){
@@ -237,8 +266,10 @@ static int realmain(int argc, char **argv){
   }
   if((rp = cbmapget(params, "max", -1, NULL)) != NULL) p_max = atoi(rp);
   if(p_max < 0) p_max = SEARCHMAX;
+  if(!(p_distinct = cbmapget(params, "distinct", -1, NULL))) p_distinct = "";
   if((rp = cbmapget(params, "fresh", -1, NULL)) != NULL) p_fresh = atoi(rp) > 0;
   if((rp = cbmapget(params, "stmode", -1, NULL)) != NULL) p_stmode = atoi(rp) > 0;
+  if(!(p_logmsg = cbmapget(params, "logmsg", -1, NULL))) p_logmsg = "";
   p_phraselist = plist;
   p_attrlist = alist;
   /* open databases */
@@ -254,15 +285,14 @@ static int realmain(int argc, char **argv){
   omode = ESTDBREADER;
   if(!g_lockindex) omode |= ESTDBNOLCK;
   for(i = 0; i < cblistnum(g_indexlist); i++){
-    rp = cblistval(p_phraselist, i, NULL);
-    if(rp && *rp != '\0'){
+    if((rp = cblistval(p_phraselist, i, NULL)) != NULL && *rp != '\0'){
       if(!g_dbs[i]){
         for(j = 0; j <= LOCKRETRYNUM; j++){
           if((g_dbs[i] = est_mtdb_open(cblistval(g_indexlist, i, NULL), omode,
                                        &ecode)) != NULL) break;
           est_usleep(1000 * 1000);
         }
-        if(!g_dbs[i]) showerror("one of the index is missing or broken or being updated.");
+        if(!g_dbs[i]) showerror("one of the indexes is missing or broken or being updated.");
         est_mtdb_set_cache_size(g_dbs[i], -1, 512, 128, 0);
         est_mtdb_set_wildmax(g_dbs[i], g_wildmax);
       }
@@ -278,12 +308,13 @@ static int realmain(int argc, char **argv){
                                      &ecode)) != NULL) break;
         est_usleep(1000 * 1000);
       }
-      if(!g_dbs[0]) showerror("one of the index is missing or broken or being updated.");
+      if(!g_dbs[0]) showerror("one of the indexes is missing or broken or being updated.");
       est_mtdb_set_cache_size(g_dbs[0], -1, 512, 128, 0);
       est_mtdb_set_wildmax(g_dbs[0], g_wildmax);
     }
     g_metamin++;
   }
+  if(g_union) g_metamin = 1;
   /* show the result */
   showresult();
   /* output the log message */
@@ -394,8 +425,8 @@ static void showresult(void){
   CBMAP *rmap;
   CBLIST *list;
   const char *rp;
-  char tmppath[PATH_MAX];
-  int i, j, num, hnum;
+  char tmppath[PATH_MAX], *expr;
+  int i, j, num, hnum, max;
   struct stat sbuf;
   time_t now;
   printf("Content-Type: text/plain\r\n");
@@ -417,18 +448,25 @@ static void showresult(void){
   rmap = cbmapopen();
   thlist = cbmalloc(cblistnum(g_indexlist) * sizeof(pthread_t));
   arglist = cbmalloc(cblistnum(g_indexlist) * sizeof(TARGSRCH));
+  max = p_max * 1.3 + 1;
   for(i = 0; i < cblistnum(g_indexlist); i++){
     arglist[i].db = g_dbs[i];
     cond = est_cond_new();
     if((rp = cblistval(p_phraselist, i, NULL)) != NULL && *rp != '\0')
       est_cond_set_phrase(cond, rp);
-    if(i == 0){
+    if(g_union || i == 0){
       for(j = 0; j < cblistnum(p_attrlist); j++){
         est_cond_add_attr(cond, cblistval(p_attrlist, j, NULL));
       }
     }
-    if(*g_ordexpr != '\0' && g_sort) est_cond_set_order(cond, g_ordexpr);
-    if(g_metamin < 2 && cblistnum(p_attrlist) < 1) est_cond_set_max(cond, p_max);
+    if(*g_ordexpr == '$'){
+      expr = cbsprintf("%s %s", g_ordexpr + 1, ESTORDNUMD);
+      est_cond_set_order(cond, expr);
+      free(expr);
+    } else if(*g_ordexpr != '\0' && g_sort){
+      est_cond_set_order(cond, g_ordexpr);
+    }
+    if(g_metamin < 2 && cblistnum(p_attrlist) < 1) est_cond_set_max(cond, max);
     switch(g_condgstep){
     case 1:
       est_cond_set_options(cond, ESTCONDSURE);
@@ -459,6 +497,7 @@ static void showresult(void){
       break;
     }
     est_cond_set_options(cond, ESTCONDSCFB);
+    if(*p_distinct != '\0') est_cond_set_distinct(cond, p_distinct);
     arglist[i].cond = cond;
     arglist[i].scores = NULL;
     arglist[i].snum = -1;
@@ -466,14 +505,14 @@ static void showresult(void){
     arglist[i].now = now;
     arglist[i].hnum = 0;
   }
-  if(g_stmode || g_metamin < 2 || p_stmode){
+  if(g_stmode || (!g_union && g_metamin < 2) || p_stmode){
     for(i = cblistnum(g_indexlist) - 1; i >= 0; i--){
       if(i < cblistnum(g_indexlist) - 1){
         arglist[i].scores = arglist[i+1].scores;
         arglist[i].snum = arglist[i+1].snum;
       }
       procsearch(arglist + i);
-      if(arglist[i].hnum == 0) break;
+      if(!g_union && arglist[i].hnum == 0) break;
     }
   } else {
     for(i = 0; i < cblistnum(g_indexlist); i++){
@@ -492,6 +531,7 @@ static void showresult(void){
   free(arglist);
   free(thlist);
   elems = est_resmap_dump(rmap, g_metamin, &num);
+  if(num < max && num < hnum) hnum = num;
   g_hnum = g_metamin < 2 ? hnum : num;
   printf("%d\n", g_hnum);
   for(i = 0; i < num && i < p_max; i++){
@@ -512,8 +552,8 @@ static void *procsearch(void *targ){
   ESTDOC *doc;
   CBMAP *rmap, *hints, *umap;
   CBLIST *list;
-  const char *rp;
-  char tmppath[PATH_MAX], *value, *enc, *pv, *mp, numbuf[NUMBUFSIZ];
+  const char *rp, *uri;
+  char tmppath[PATH_MAX], *value, *expr, *ord, *enc, *pv, *mp, numbuf[NUMBUFSIZ];
   const int *scores;
   int i, *res, rnum, snum, check, len, num;
   struct stat sbuf;
@@ -568,21 +608,27 @@ static void *procsearch(void *targ){
           *pv = '\0';
           if((mp = strchr(rp, '\t')) != NULL){
             *mp = '\0';
-            est_resmap_add(rmap, rp, atoi(mp + 1));
+            est_resmap_add(rmap, rp, atoi(mp + 1), ESTRMLOMAX);
           }
           rp = pv + 1;
         }
         pthread_mutex_unlock(&mymutex);
       }
       free(value);
-      if(g_cclife >= 0) est_utime(tmppath, sbuf.st_mtime + (now - sbuf.st_mtime) / 2);
+      if(g_cclife >= 0){
+        if(now % 4 == 0 && getpid() % 4 == 0 && est_random() < 0.1){
+          unlink(tmppath);
+        } else {
+          est_utime(tmppath, sbuf.st_mtime + (now - sbuf.st_mtime) / 2);
+        }
+      }
       g_cache = TRUE;
       return NULL;
     } else {
       unlink(tmppath);
     }
   }
-  if(argp->scores && *g_idattr == '\0')
+  if(argp->scores && *g_idattr == '\0' && !g_union)
     est_cond_set_narrowing_scores(cond, argp->scores, argp->snum);
   hints = cbmapopenex(MINIBNUM);
   etime = est_gettimeofday();
@@ -592,9 +638,11 @@ static void *procsearch(void *targ){
     argp->scores = scores;
     argp->snum = snum;
   }
-  check = (rp = est_cond_phrase(cond)) != NULL && *rp != '\0' && *rp != '[' ? g_scancheck : 0;
+  check = (rp = est_cond_phrase(cond)) != NULL && *rp != '\0' && *rp != '[' && *rp != '*' ?
+    g_scancheck : 0;
   ofp = *tmppath != '\0' && etime >= CCGENMINTIME ? fopen(tmppath, "w") : NULL;
   num = (rp = cbmapget(hints, "", 0, NULL)) != NULL ? atoi(rp) : 0;
+  if(rnum < est_cond_max(cond) && num < rnum) num = rnum;
   argp->hnum += num;
   if(ofp) fprintf(ofp, "%d\n", num);
   if(pthread_mutex_lock(&mymutex) == 0){
@@ -613,13 +661,41 @@ static void *procsearch(void *targ){
         if((num = est_cond_score(cond, i)) > 0){
           sprintf(numbuf, "%d", num);
           if(!umap || cbmaprnum(umap) > DUPCHKMAX || cbmapput(umap, numbuf, -1, "", 0, FALSE)){
-            est_resmap_add(rmap, numbuf, num);
-            if(ofp) fprintf(ofp, "%s\t%d\n", numbuf, num);
+            if(cblistnum(g_idsuflist) > 0){
+              expr = caturisuffixes(db, res[i], numbuf);
+              uri = expr;
+            } else {
+              expr = NULL;
+              uri = numbuf;
+            }
+            if(*g_ordexpr == '$'){
+              ord = est_mtdb_get_doc_attr(db, res[i], g_ordexpr + 1);
+              num = ord ? cbstrmktime(ord) : 0;
+              free(ord);
+            }
+            est_resmap_add(rmap, uri, num, g_score);
+            if(ofp) fprintf(ofp, "%s\t%d\n", uri, num);
+            free(expr);
           }
         } else if((value = est_mtdb_get_doc_attr(db, res[i], ESTDATTRURI)) != NULL){
           if(!umap || cbmaprnum(umap) > DUPCHKMAX || cbmapput(umap, value, -1, "", 0, FALSE)){
-            est_resmap_add(rmap, value, atoi(value));
-            if(ofp) fprintf(ofp, "%s\t%s\n", value, value);
+            if(cblistnum(g_idsuflist) > 0){
+              expr = caturisuffixes(db, res[i], value);
+              uri = expr;
+            } else {
+              expr = NULL;
+              uri = value;
+            }
+            if(*g_ordexpr == '$'){
+              ord = est_mtdb_get_doc_attr(db, res[i], g_ordexpr + 1);
+              num = ord ? cbstrmktime(ord) : 0;
+              free(ord);
+            } else {
+              num = atoi(value);
+            }
+            est_resmap_add(rmap, uri, num, g_score);
+            if(ofp) fprintf(ofp, "%s\t%d\n", uri, num);
+            free(expr);
           }
           free(value);
         }
@@ -634,15 +710,43 @@ static void *procsearch(void *targ){
           }
           if((rp = est_doc_attr(doc, g_idattr)) != NULL){
             num = est_cond_score(cond, i);
-            est_resmap_add(rmap, rp, num);
-            if(ofp) fprintf(ofp, "%s\t%d\n", rp, num);
+            if(cblistnum(g_idsuflist) > 0){
+              expr = caturisuffixes(db, res[i], rp);
+              uri = expr;
+            } else {
+              expr = NULL;
+              uri = rp;
+            }
+            if(*g_ordexpr == '$'){
+              ord = est_mtdb_get_doc_attr(db, res[i], g_ordexpr + 1);
+              num = ord ? cbstrmktime(ord) : 0;
+              free(ord);
+            }
+            est_resmap_add(rmap, uri, num, g_score);
+            if(ofp) fprintf(ofp, "%s\t%d\n", uri, num);
+            free(expr);
           }
           est_doc_delete(doc);
           check--;
         } else if((value = est_mtdb_get_doc_attr(db, res[i], g_idattr)) != NULL){
           num = est_cond_score(cond, i);
-          est_resmap_add(rmap, value, num);
-          if(ofp) fprintf(ofp, "%s\t%d\n", value, num);
+          if(cblistnum(g_idsuflist) > 0){
+            expr = caturisuffixes(db, res[i], value);
+            uri = expr;
+          } else {
+            expr = NULL;
+            uri = value;
+          }
+          if(*g_ordexpr == '$'){
+            ord = est_mtdb_get_doc_attr(db, res[i], g_ordexpr + 1);
+            num = ord ? cbstrmktime(ord) : 0;
+            free(ord);
+          } else if(num < 1){
+            num = atoi(value);
+          }
+          est_resmap_add(rmap, uri, num, g_score);
+          if(ofp) fprintf(ofp, "%s\t%d\n", uri, num);
+          free(expr);
           free(value);
         }
       }
@@ -708,6 +812,23 @@ static char *myencode(ESTCOND *cond){
 }
 
 
+/* concatenate suffixes after URI */
+static char *caturisuffixes(ESTMTDB *db, int id, const char *uri){
+  CBDATUM *buf;
+  char *value;
+  int i;
+  buf = cbdatumopen(uri, -1);
+  for(i = 0; i < cblistnum(g_idsuflist); i++){
+    cbdatumcat(buf, "@", 1);
+    if((value = est_mtdb_get_doc_attr(db, id, cblistval(g_idsuflist, i, NULL))) != NULL){
+      cbdatumcat(buf, value, -1);
+      free(value);
+    }
+  }
+  return cbdatumtomalloc(buf, NULL);
+}
+
+
 /* output the log message */
 static void outputlog(void){
   FILE *ofp;
@@ -774,6 +895,9 @@ static void outputlog(void){
       } else if(cbstrfwmatch(rp, "{metamin}")){
         fprintf(ofp, "%d", g_metamin);
         rp += 8;
+      } else if(cbstrfwmatch(rp, "{logmsg}")){
+        fprintf(ofp, "%s", p_logmsg);
+        rp += 7;
       } else if((pv = strchr(rp, '}')) != NULL){
         rp++;
         name = cbmemdup(rp, pv - rp);
