@@ -796,9 +796,9 @@ static void showpage(void){
     }
   }
   if(!strcmp(p_order, "_date_")){
-    est_cond_set_order(cond, ESTDATTRMDATE " NUMD");
+    est_cond_set_order(cond, ESTDATTRMDATE " " ESTORDNUMD);
   } else if(!strcmp(p_order, "_size_")){
-    est_cond_set_order(cond, ESTDATTRSIZE " NUMD");
+    est_cond_set_order(cond, ESTDATTRSIZE " " ESTORDNUMD);
   } else if(p_order[0] != '\0'){
     est_cond_set_order(cond, p_order);
   }
@@ -853,7 +853,7 @@ static void showpage(void){
     docs = cbmalloc(rnum * sizeof(ESTDOC *) + 1);
     dnum = 0;
     miss = 0;
-    check = p_phrase[0] == '\0' || p_phrase[0] == '[' ? 0 : g_scancheck;
+    check = p_phrase[0] == '\0' || p_phrase[0] == '[' || p_phrase[0] == '*' ? 0 : g_scancheck;
     for(i = 0; i < rnum; i++){
       docs[dnum] = est_db_get_doc(g_db, res[i],
                                   dnum < p_pagenum * p_perpage || check > 0 ? 0 :
