@@ -44,6 +44,7 @@ int g_runcnt = 0;                        /* count of running */
 const char *g_conffile = NULL;           /* path of the configuration file */
 const CBLIST *g_targetlist = NULL;       /* list of real indexes */
 int g_stmode = FALSE;                    /* whether to be single thread mode */
+int g_score = -1;                        /* scoring method when logical operation */
 const char *g_tmpdir = NULL;             /* path of the directory for temporary files */
 int g_cclife = 0;                        /* lifetime of cache files */
 const CBLIST *g_sharelist = NULL;        /* list of cache sharing hosts */
@@ -57,10 +58,12 @@ const char *p_querykey = "";             /* unique key for the query string */
 const char *p_shareurl = "";             /* URL of shared cache */
 const CBLIST *p_phraselist = NULL;       /* list of phrase conditions */
 const CBLIST *p_attrlist = NULL;         /* list of attribute conditions */
+const char *p_distinct = NULL;           /* name of distinction attribute */
 int p_max = -1;                          /* number of output documents */
 int p_fresh = FALSE;                     /* whether to retrieve fresh result */
 int p_stmode = FALSE;                    /* whether to be single thread mode */
 const char *p_cache = "";                /* key of read cache */
+const char *p_logmsg = "";               /* additional log message */
 
 
 /* other global variables */
@@ -99,9 +102,11 @@ int main(int argc, char **argv){
     p_phraselist = NULL;
     p_attrlist = NULL;
     p_max = -1;
+    p_distinct = NULL;
     p_fresh = FALSE;
     p_stmode = FALSE;
     p_cache = "";
+    p_logmsg = "";
     g_scripturl = NULL;
     g_hnum = 0;
     g_cache = FALSE;
@@ -162,6 +167,8 @@ static int realmain(int argc, char **argv){
       if(*rp != '\0') cblistpush(tlist, rp, -1);
     } else if(cbstrfwimatch(rp, "stmode:")){
       g_stmode = atoi(skiplabel(rp)) > 0;
+    } else if(cbstrfwimatch(rp, "score:")){
+      g_score = atoi(skiplabel(rp));
     } else if(cbstrfwimatch(rp, "failfile:")){
       g_failfile = skiplabel(rp);
     } else if(cbstrfwimatch(rp, "tmpdir:")){
@@ -179,6 +186,7 @@ static int realmain(int argc, char **argv){
   }
   if(cblistnum(tlist) < 1) showerror("targeturl is undefined.");
   g_targetlist = tlist;
+  if(g_score < 0) showerror("score is undefined.");
   if(!g_failfile) showerror("failfile is undefined.");
   if(!g_tmpdir) showerror("tmpdir is undefined.");
   g_sharelist = slist;
@@ -221,7 +229,6 @@ static int realmain(int argc, char **argv){
       }
       cblistpush(plist, rp, -1);
     }
-    if((rp = cbmapget(params, "attr", -1, NULL)) != NULL) cblistpush(alist, rp, -1);
     for(i = 1; i <= CONDATTRMAX; i++){
       len = sprintf(numbuf, "attr%d", i);
       rp = cbmapget(params, numbuf, len, NULL);
@@ -234,9 +241,11 @@ static int realmain(int argc, char **argv){
     }
     if((rp = cbmapget(params, "max", -1, NULL)) != NULL) p_max = atoi(rp);
     if(p_max < 0) p_max = SEARCHMAX;
+    if(!(p_distinct = cbmapget(params, "distinct", -1, NULL))) p_distinct = "";
     if(!(p_cache = cbmapget(params, "cache", -1, NULL))) p_cache = "";
     if((rp = cbmapget(params, "fresh", -1, NULL)) != NULL) p_fresh = atoi(rp) > 0;
     if((rp = cbmapget(params, "stmode", -1, NULL)) != NULL) p_stmode = atoi(rp) > 0;
+    if(!(p_logmsg = cbmapget(params, "logmsg", -1, NULL))) p_logmsg = "";
     p_phraselist = plist;
     p_attrlist = alist;
     if(*p_cache != '\0' && *g_tmpdir != '\0'){
@@ -354,7 +363,7 @@ static CBMAP *getparameters(void){
     cbglobalgc(key, free);
     p_querykey = key;
     if(cblistnum(g_sharelist) > 0) p_shareurl =
-                                     cblistval(g_sharelist, dpouterhash(p_querykey, -1) % cblistnum(g_sharelist), NULL);
+      cblistval(g_sharelist, dpouterhash(p_querykey, -1) % cblistnum(g_sharelist), NULL);
     if((rp = getenv("CONTENT_TYPE")) != NULL && cbstrfwmatch(rp, "multipart/form-data") &&
        (rp = strstr(rp, "boundary=")) != NULL){
       rp += 9;
@@ -529,7 +538,13 @@ static void showresult(void){
        (str = cbreadfile(tmppath, &len)) != NULL){
       g_hnum = atoi(str);
       fwrite(str, 1, len, stdout);
-      if(g_cclife >= 0) est_utime(tmppath, sbuf.st_mtime + (now - sbuf.st_mtime) / 2);
+      if(g_cclife >= 0){
+        if(now % 4 == 0 && getpid() % 4 == 0 && est_random() < 0.1){
+          unlink(tmppath);
+        } else {
+          est_utime(tmppath, sbuf.st_mtime + (now - sbuf.st_mtime) / 2);
+        }
+      }
       free(str);
       g_cache = TRUE;
       return;
@@ -580,6 +595,7 @@ static void showresult(void){
   free(arglist);
   free(thlist);
   elems = est_resmap_dump(rmap, 1, &num);
+  if(num < p_max + 1 && num < hnum) hnum = num;
   g_hnum = hnum;
   obuf = cbdatumopen(NULL, -1);
   cbdatumprintf(obuf, "%d\n", g_hnum);
@@ -611,7 +627,7 @@ static void *procsearch(void *targ){
   CBLIST *fields;
   CBDATUM *url, *body;
   const char *rp, *pv, *key;
-  int i, code, num;
+  int i, code, num, rnum;
   argp = (TARGSRCH *)targ;
   url = cbdatumopen(argp->url, -1);
   rmap = argp->rmap;
@@ -624,9 +640,11 @@ static void *procsearch(void *targ){
     rp = cblistval(p_attrlist, i, NULL);
     if(*rp != '\0') cbdatumprintf(url, "attr%d=%?&", i + 1, rp);
   }
-  cbdatumprintf(url, "max=%d", p_max);
+  cbdatumprintf(url, "max=%d", p_max + 1);
+  if(*p_distinct != '\0') cbdatumprintf(url, "&distinct=%?", p_distinct);
   if(p_fresh) cbdatumprintf(url, "&fresh=1");
   if(p_stmode) cbdatumprintf(url, "&stmode=1");
+  if(*p_logmsg) cbdatumprintf(url, "&logmsg=%?", p_logmsg);
   body = cbdatumopen(NULL, -1);
   if(est_url_shuttle(cbdatumptr(url), NULL, -1, -1, -1, NULL, NULL, NULL, -1,
                      &code, NULL, body) && code == 200){
@@ -635,16 +653,21 @@ static void *procsearch(void *targ){
     if(num > 0 && (rp = strchr(rp, '\n')) != NULL){
       argp->hnum = num;
       if(pthread_mutex_lock(&mymutex) == 0){
+        rnum = 0;
         while((pv = strchr(rp, '\n')) != NULL){
           fields = cbsplit(rp, pv - rp, "\t");
           if(cblistnum(fields) > 1){
             key = cblistval(fields, 0, NULL);
             num = atoi(cblistval(fields, 1, NULL));
-            if(*key != '\0') est_resmap_add(rmap, key, num);
+            if(*key != '\0'){
+              est_resmap_add(rmap, key, num, g_score);
+              rnum++;
+            }
           }
           cblistclose(fields);
           rp = pv + 1;
         }
+        if(rnum < p_max + 1 && rnum < argp->hnum) argp->hnum = rnum;
         pthread_mutex_unlock(&mymutex);
       }
     }
@@ -726,6 +749,9 @@ static void outputlog(void){
       } else if(cbstrfwmatch(rp, "{speed}")){
         fprintf(ofp, "%.3f", g_speed / 1000);
         rp += 6;
+      } else if(cbstrfwmatch(rp, "{logmsg}")){
+        fprintf(ofp, "%s", p_logmsg);
+        rp += 7;
       } else if((pv = strchr(rp, '}')) != NULL){
         rp++;
         name = cbmemdup(rp, pv - rp);
