@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The command line interface of web crawler
- *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2007 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -115,7 +115,7 @@ static void usage(void){
   fprintf(stderr, "%s: command line interface of web crawler\n", g_progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s init [-xs|-xl|-xh] rootdir\n", g_progname);
+  fprintf(stderr, "  %s init [-apn|-acc] [-xs|-xl|-xh] [-sv|-si|-sa] rootdir\n", g_progname);
   fprintf(stderr, "  %s crawl [-restart|-revisit|-revcont] rootdir\n", g_progname);
   fprintf(stderr, "  %s unittest rootdir\n", g_progname);
   fprintf(stderr, "  %s fetch [-proxy host port] [-tout num] [-il lang] url\n", g_progname);
@@ -352,12 +352,22 @@ static int runinit(int argc, char **argv){
   opts = 0;
   for(i = 2; i < argc; i++){
     if(!rootdir && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-xs")){
+      if(!strcmp(argv[i], "-apn")){
+        opts |= WI_PERFNG;
+      } else if(!strcmp(argv[i], "-acc")){
+        opts |= WI_CHRCAT;
+      } else if(!strcmp(argv[i], "-xs")){
         opts |= WI_SMALL;
       } else if(!strcmp(argv[i], "-xl")){
         opts |= WI_LARGE;
       } else if(!strcmp(argv[i], "-xh")){
         opts |= WI_HUGE;
+      } else if(!strcmp(argv[i], "-sv")){
+        opts |= WI_SCVOID;
+      } else if(!strcmp(argv[i], "-si")){
+        opts |= WI_SCINT;
+      } else if(!strcmp(argv[i], "-sa")){
+        opts |= WI_SCASIS;
       } else {
         usage();
       }
@@ -553,8 +563,8 @@ static int proccrawl(const char *rootdir, int mode){
   kwbuf = cbdatumopen(NULL, -1);
   cbmapiterinit(waver->kwords);
   for(i = 0; (kbuf = cbmapiternext(waver->kwords, NULL)) != NULL; i++){
-    if(i > 0) est_datum_printf(kwbuf, ", ");
-    est_datum_printf(kwbuf, "%s (%s)", kbuf, cbmapget(waver->kwords, kbuf, -1, NULL));
+    if(i > 0) cbdatumprintf(kwbuf, ", ");
+    cbdatumprintf(kwbuf, "%s (%s)", kbuf, cbmapget(waver->kwords, kbuf, -1, NULL));
   }
   log_print(LL_DEBUG, "seed keywords: %s", cbdatumptr(kwbuf));
   cbdatumclose(kwbuf);
@@ -1133,13 +1143,13 @@ static char *urltosavepath(const char *savedir, const char *url){
   buf = cbdatumopen(NULL, -1);
   elems = cbsplit(url, -1, "/");
   if(cbstrbwmatch(url, "/")) cblistpush(elems, "index.html", -1);
-  est_datum_printf(buf, "%s", savedir);
+  cbdatumprintf(buf, "%s", savedir);
   for(i = 0; i < cblistnum(elems); i++){
     rp = cblistval(elems, i, NULL);
     if(rp[0] == '\0') continue;
     est_mkdir(cbdatumptr(buf));
     unlink(cbdatumptr(buf));
-    est_datum_printf(buf, "%c%?", ESTPATHCHR, rp);
+    cbdatumprintf(buf, "%c%?", ESTPATHCHR, rp);
   }
   cblistclose(elems);
   return cbdatumtomalloc(buf, NULL);
