@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of wavermod
- *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2007 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -169,10 +169,10 @@ int waver_init(const char *rootdir, int options){
   sprintf(path, "%s%c%s", rootdir, ESTPATHCHR, CONFFILE);
   if((ofp = fopen(path, "wb")) != NULL){
     fprintf(ofp, "# seed documents (weight and URL)\n");
-    fprintf(ofp, "seed: 1.5|http://hyperestraier.sourceforge.net/uguide-en.html\n");
-    fprintf(ofp, "seed: 1.0|http://hyperestraier.sourceforge.net/pguide-en.html\n");
-    fprintf(ofp, "seed: 1.0|http://hyperestraier.sourceforge.net/nguide-en.html\n");
-    fprintf(ofp, "seed: 0.0|http://qdbm.sourceforge.net/\n");
+    fprintf(ofp, "seed: 1.5|http://fallabs.com/hyperstraier/uguide-en.html\n");
+    fprintf(ofp, "seed: 1.0|http://fallabs.com/hyperestraier/pguide-en.html\n");
+    fprintf(ofp, "seed: 1.0|http://fallabs.com/hyperestraier/nguide-en.html\n");
+    fprintf(ofp, "seed: 0.0|http://fallabs.com/qdbm/\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# host name of the proxy\n");
     fprintf(ofp, "proxyhost:\n");
@@ -292,15 +292,27 @@ int waver_init(const char *rootdir, int options){
   }
   tracebnum = TRACEBNUM;
   estopts = 0;
+  if(options & WI_PERFNG){
+    estopts |= ESTDBPERFNG;
+  } else if(options & WI_CHRCAT){
+    estopts |= ESTDBCHRCAT;
+  }
   if(options & WI_SMALL){
     tracebnum = TRACEBNUM / 2;
-    estopts = ESTDBSMALL;
+    estopts |= ESTDBSMALL;
   } else if(options & WI_LARGE){
     tracebnum = TRACEBNUM * 2;
-    estopts = ESTDBLARGE;
+    estopts |= ESTDBLARGE;
   } else if(options & WI_HUGE){
     tracebnum = TRACEBNUM * 4;
-    estopts = ESTDBHUGE;
+    estopts |= ESTDBHUGE;
+  }
+  if(options & WI_SCVOID){
+    estopts |= ESTDBSCVOID;
+  } else if(options & WI_SCINT){
+    estopts |= ESTDBSCINT;
+  } else if(options & WI_SCASIS){
+    estopts |= ESTDBSCASIS;
   }
   sprintf(path, "%s%c%s", rootdir, ESTPATHCHR, QUEUEFILE);
   unlink(path);
